unit SDCCIniFile_U;
// This file is part of Sarah Dean's Code Co-op Shell Extension.
//
//  "Sarah Dean's Code Co-op Shell Extension" is free software; you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either
// version 2 of the License, or at your option) any later version.
//
//  "Sarah Dean's Code Co-op Shell Extension" is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with "Sarah Dean's Code Co-op Shell Extension"; if not, write to
// the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
// MA  02111-1307  USA


// Description: SecureTrayUtil INI File
// By Sarah Dean
//
// Defines INI file contents and methods for reading and writing them
//
// -----------------------------------------------------------------------------


interface

uses
     registry,
     dialogs,
     forms,
     classes,
     sysutils,
     Windows, // needed for WritePrivateProfileString
     SDCCIniFileConsts_U,
     AppGlobalConsts_U;

type

  TSDCCIniFile = class(TRegINIFile)
  private
    function ReadIniFileGeneral() : boolean;
    function ReadIniFileShellExt() : boolean;

    function WriteIniFileGeneral() : boolean;
    function WriteIniFileShellExt() : boolean;
  public
    NewVersion: boolean; // Set if this if there is no INI file or if it's for a previous version

    // General options
    OptShellExtSubMenu         : boolean;
    OptShellExtSubMenuTitle    : string;
    OptShellExtMenuSeparators  : boolean;
    OptShellExtConfirmCheckouts: boolean;

    constructor Create(const FileName : string);
    destructor  Destroy(); override;

    function ReadIniFile() : boolean;
    function WriteIniFile() : boolean;

    procedure CheckVersionID();

  end;

implementation

uses
  SDUGeneral;

constructor TSDCCIniFile.Create(const FileName : string);
begin
  inherited;
end;

destructor TSDCCIniFile.Destroy();
begin
  inherited;
end;

function TSDCCIniFile.ReadIniFile() : boolean;
var
  retVal : boolean;
begin
  retVal := ReadIniFileGeneral();
  retVal := retVal AND ReadIniFileShellExt();

  Result := retVal;
end;

function TSDCCIniFile.ReadIniFileGeneral() : boolean;
begin
  Result := TRUE;
end;



function TSDCCIniFile.ReadIniFileShellExt() : boolean;
begin
  OptShellExtSubMenu         := ReadBool(SECTION_CODECOOPSHELLEXT, KEY_SUBMENU, DFLT_KEY_SUBMENU);
  OptShellExtSubMenuTitle    := ReadString(SECTION_CODECOOPSHELLEXT, KEY_SUBMENUTITLE, DFLT_KEY_SUBMENUTITLE);
  OptShellExtMenuSeparators  := ReadBool(SECTION_CODECOOPSHELLEXT, KEY_MENUITEM_SEPARATORS, DFLT_KEY_MENUITEM_SEPARATORS);
  OptShellExtConfirmCheckouts:= ReadBool(SECTION_CODECOOPSHELLEXT, KEY_CONFIRMCHECKOUTS, DFLT_KEY_CONFIRMCHECKOUTS);

  Result := TRUE;
end;


function TSDCCIniFile.WriteIniFile() : boolean;
var
  retVal : boolean;
begin
  retVal := WriteIniFileGeneral();
  retVal := retVal AND WriteIniFileShellExt();

  Result := retVal;
end;

function TSDCCIniFile.WriteIniFileGeneral() : boolean;
var
  thisMajorVersion   : integer;
  thisMinorVersion   : integer;
  thisRevisionVersion: integer;
  thisBuildVersion   : integer;

  strVersion: string;
begin
  SDUGetVersionInfo('', thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion);
  strVersion := SDUGetVersionInfoString('');
  if (APP_BETA_BUILD>-1) then
    begin
    strVersion := strVersion + ' BETA ' + inttostr(APP_BETA_BUILD);
    end;

  WriteString(SECTION_CODECOOPSHELLEXT, KEY_VERSION, strVersion);
  WriteInteger(SECTION_CODECOOPSHELLEXT, KEY_MAJOR_VERSION,   thisMajorVersion);
  WriteInteger(SECTION_CODECOOPSHELLEXT, KEY_MINOR_VERSION,   thisMinorVersion);
  WriteInteger(SECTION_CODECOOPSHELLEXT, KEY_RELEASE_VERSION, thisRevisionVersion);
  WriteInteger(SECTION_CODECOOPSHELLEXT, KEY_BUILD_VERSION,   thisBuildVersion);
  WriteInteger(SECTION_CODECOOPSHELLEXT, KEY_BETA_VERSION,    APP_BETA_BUILD);

  Result := TRUE;
end;


function TSDCCIniFile.WriteIniFileShellExt() : boolean;
begin
  WriteBool(SECTION_CODECOOPSHELLEXT, KEY_SUBMENU, OptShellExtSubMenu);
  WriteString(SECTION_CODECOOPSHELLEXT, KEY_SUBMENUTITLE, OptShellExtSubMenuTitle);
  WriteBool(SECTION_CODECOOPSHELLEXT, KEY_MENUITEM_SEPARATORS, OptShellExtMenuSeparators);
  WriteBool(SECTION_CODECOOPSHELLEXT, KEY_CONFIRMCHECKOUTS, OptShellExtConfirmCheckouts);

  Result := TRUE;
end;


procedure TSDCCIniFile.CheckVersionID();
var
  oldVersionStr: string;
  upgradeMessage: string;

  majorVersion   : integer; // Only used to build up oldVersionStr
  minorVersion   : integer; // Only used to build up oldVersionStr
  revisionVersion: integer; // Only used to build up oldVersionStr
  buildVersion   : integer; // Only used to build up oldVersionStr

  thisMajorVersion   : integer;
  thisMinorVersion   : integer;
  thisRevisionVersion: integer;
  thisBuildVersion   : integer;
  strThisVersion     : string;

  betaVersion: integer;
begin
  oldVersionStr    := ReadString(SECTION_CODECOOPSHELLEXT, KEY_VERSION, '-1.0');
  majorVersion     := ReadInteger(SECTION_CODECOOPSHELLEXT, KEY_MAJOR_VERSION, -1);
  minorVersion     := ReadInteger(SECTION_CODECOOPSHELLEXT, KEY_MINOR_VERSION, -1);
  revisionVersion  := ReadInteger(SECTION_CODECOOPSHELLEXT, KEY_RELEASE_VERSION, -1);
  buildVersion     := ReadInteger(SECTION_CODECOOPSHELLEXT, KEY_BUILD_VERSION, -1);
  betaVersion      := ReadInteger(SECTION_CODECOOPSHELLEXT, KEY_BETA_VERSION, -1);

  SDUGetVersionInfo('', thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion);
  strThisVersion := Format('%d.%.2d.%.2d.%.4d', [thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion]);
  if (APP_BETA_BUILD>-1) then
    begin
    strThisVersion := strThisVersion + ' BETA ' + inttostr(APP_BETA_BUILD);
    end;

  NewVersion := FALSE;

  if oldVersionStr='-1.0' then
    begin
    // Any message to be displayed the 1st time the user runs the application
    // to be placed here
    NewVersion := TRUE;
    end
  else if (oldVersionStr='1.00') OR
          (oldVersionStr='1.10') OR
          (majorVersion<thisMajorVersion) OR
          ((majorVersion=thisMajorVersion) AND (minorVersion<thisMinorVersion)) OR
          ((betaVersion<>-1) AND (not(APP_BETA_BUILD>-1))) OR
          ((betaVersion<>-1) AND (betaVersion<APP_BETA_BUILD)) then
    begin
    // The INI file dates back from an older version, so warn the user
    
    if not((oldVersionStr='1.00') or (oldVersionStr='1.10')) then
      begin
      oldVersionStr := Format('%d.%.2d.%.2d.%.4d', [majorVersion, minorVersion, revisionVersion, buildVersion]);
      end;

    if betaVersion>-1 then
      begin
      oldVersionStr := oldVersionStr + ' BETA ' + inttostr(betaVersion);
      end;

    upgradeMessage := 'It appears as though your configuration dates back to v'+oldVersionStr;
    upgradeMessage := upgradeMessage+CRLF+CRLF+'You may find that some of your options mysteriously revert back to their defaults when running this version (v'+strThisVersion+').';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'This is "normal", and due to two things:';
    upgradeMessage := upgradeMessage+CRLF+'1) the format of the ini file has changed to make it more logical and to take into account improvements in the software';
    upgradeMessage := upgradeMessage+CRLF+'2) lazyness on the part of the author (it wouldn''t be difficult to write a converter, but it''s just too much hassle)';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'To prevent this dialog from reappearing, either delete your existing SecureTrayUtil.ini file (located in your Windows directory) or goto any of the options screens, change an option, change it back, and click "OK" or "Apply"';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'It is recommended that you check your option settings to ensure correct operation.';

    messagedlg(upgradeMessage, mtWarning, [mbOK], 0);

    NewVersion := TRUE;
    end;

end;


END.

